#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "fitsio.h"

#define LINEPIX 2048
#define MAXCHAR 2048
#define MAXHDR 2048
#define HDRSIZE 100

main(int argc, char **argv)
{
  FILE *fp;
  fitsfile *fptr1,*fptr2;
  int status, nfound, anynull;
  long fpixel, npixels;
  int i,j;
  float nullval; 
  char openfile[100],savefile[100],paramfile[100];

  int bitpix=-32;
  long naxis=2;

  long naxes[2]={LINEPIX, LINEPIX};
  static float buffer[LINEPIX],pix[LINEPIX][LINEPIX];
  long nbuffer=LINEPIX;

  char line[MAXCHAR];
  char disperser[MAXCHAR];
 
  int nkeys;
  char *hdrptr[MAXHDR];

  int direction, niter, iter, nsum;
  float nsigma, sum, sum2;
  static float sigma[LINEPIX], av[LINEPIX];
  
  
  if(argc !=6){
    fprintf(stderr,"Usage : %s (input file) (output file) (x/y) (# of iteration) (clipping nsigma) \n",argv[0]);
    exit(1);
  }

  sprintf(openfile, "%s",argv[1]);
  sprintf(savefile, "%s",argv[2]);
  if(!strcmp(argv[3],"x")) direction=0;
  if(!strcmp(argv[3],"y")) direction=1;
  niter=atoi(argv[4]);
  nsigma=atof(argv[5]);

  printf("Compress in %s direction(%d)\n",argv[3],direction);
  printf("%s \t: Read %s  ... ",argv[0],openfile);

  status=0;

  if(fits_open_file(&fptr1,openfile, READONLY, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /* Read Header Information */
  if(fits_read_keys_lng(fptr1, "NAXIS", 1, 2, naxes, &nfound, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  if((naxes[0]>LINEPIX)||(naxes[1]>LINEPIX)){
    fprintf(stderr,"Image is too large\n");
    exit(0);
  }
  printf("%d x %d pixels\n",naxes[0],naxes[1]);

  if(fits_get_hdrspace(fptr1, &nkeys, NULL, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  for (i=0;i<nkeys;i++){
    hdrptr[i]=malloc(HDRSIZE);
    if(fits_read_record(fptr1,i+1,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  /* naxes[0] :  x */
  /* naxes[1] : Կ y */
  /* j :  x*/
  /* i : Կ y*/
  /* pix[y][x] */

  fpixel = 1;
  nullval = 0;
  nbuffer=naxes[0];
  for(i=0;i<naxes[1];i++){
    if (fits_read_img(fptr1, TFLOAT, fpixel, nbuffer, &nullval,
                      buffer, &anynull, &status)){
      fits_report_error(stderr,status);
      return(status);
    }
    for(j=0;j<naxes[0];j++){
      pix[i][j]=buffer[j];
    }
    fpixel+=nbuffer;
  }

  if(fits_close_file(fptr1, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("Read OK\n");

  for(iter=0;iter<niter;iter++){
    if(direction==0){
      for(i=0;i<naxes[1];i++){
	sum=0;
	sum2=0;
	nsum=0;
	for(j=0;j<naxes[0];j++){
	  if(iter==0 || (pix[i][j]>av[i]-nsigma*sigma[i] && pix[i][j]<av[i]+nsigma*sigma[i] )){
	    sum+=pix[i][j];
	    sum2+=(pix[i][j]*pix[i][j]);
	    nsum++;
	  }
	}
	av[i]=sum/nsum;
	sigma[i]=sqrt(sum2/nsum-av[i]*av[i]);
	if(iter==niter-1){
	  for(j=0;j<naxes[0];j++)pix[i][j]=av[i];
	}
      }
    }

    if(direction==1){
      for(j=0;j<naxes[0];j++){
	sum=0;
	sum2=0;
	nsum=0;
	for(i=0;i<naxes[1];i++){
	  if(iter==0 || (pix[i][j]>av[j]-nsigma*sigma[j] && pix[i][j]<av[j]+nsigma*sigma[j] )){
	    sum+=pix[i][j];
	    sum2+=(pix[i][j]*pix[i][j]);
	    nsum++;
	  }
	}
	av[j]=sum/nsum;
	sigma[j]=sqrt(sum2/nsum-av[j]*av[j]);
	if(iter==niter-1){
	  for(i=0;i<naxes[1];i++)pix[i][j]=av[j];
	}
      }
    }

  }

  printf("\t: Write %s  ... ",savefile);
  unlink(savefile);

  status=0;

  if (fits_create_file(&fptr2, savefile, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  /*  if (fits_create_img(fptr2, bitpix, naxis, naxes, &status)){
    fits_report_error(stderr,status);
    return(status);
  }
  */
  for (i=0;i<nkeys;i++){
    if(fits_write_record(fptr2,hdrptr[i],&status)){
      fits_report_error(stderr,status);
      return(status);
    }
  }

  fpixel=1;

  for(i=0;i<naxes[1];i++){
    if (fits_write_img(fptr2, TFLOAT, fpixel, nbuffer, pix[i], &status)) 
      fits_report_error(stderr,status);
    fpixel+=nbuffer;
  }

  if (fits_close_file(fptr2, &status)){
    fits_report_error(stderr,status);
    return(status);
  }

  printf("OK\n");

}
